==> ask-cli-1.5.0.tgz: package/lib/dialog/simulation-sender.js <==
'use strict';

const apiWrapper = require('../api/api-wrapper.js');
const tools = require('../utils/tools');
const CONSTANTS = require('../utils/constants');
const displayUtil = require('../utils/display-utility');
const asyncWriter = require('./file-writer');
const retry = require('retry');
const jsonUtility = require('../utils/json-utility');

module.exports = {
    sendDialogSimulate: sendDialogSimulate
};

function ErrorResponse(msg, payload) {
    this.msg = msg;
    this.payload = payload;
    return this;
}

function sendDialogSimulate(text, skillId, locale, newSession, outputWriteStream, inputTextArray, profile, debug) {
    return new Promise((resolve, reject) => {
        const sendSimulateSpinner = displayUtil.Spinner('');

        asyncWriter.writeUserSays(outputWriteStream, text);

        inputTextArray.push(text);

        const operation = retry.operation({
            retries: CONSTANTS.CONFIGURATION.RETRY.SEND_SIMULATE.MAX_RETRY,
            minTimeout: CONSTANTS.CONFIGURATION.RETRY.SEND_SIMULATE.MIN_TIME_OUT,
            factor: CONSTANTS.CONFIGURATION.RETRY.SEND_SIMULATE.FACTOR
        });

        sendSimulateSpinner.start();

        let isPostRetryFinished = false;
        let callSimulationResponse = null;
        operation.attempt(() => {
            if (operation.retry(isPostRetryFinished ? null : 'needRetry')) {
                apiWrapper.callSimulateSkill(null, text, skillId, locale, newSession, profile, debug, data => {
                    callSimulationResponse = data;
                    // statusCode 400 is Bad Request which doesn't need to retry
                    // all statusCode >= 300 and not 400 will keep retry 
                    if (data.statusCode !== 400 && data.statusCode < 300) {
                        isPostRetryFinished = true;
                        const response = tools.convertDataToJsonObject(data.body);

                        if (response) {
                            pollingSimulationResult(response.id, skillId, profile, debug)
                            .then(pollingResponse => {
                                sendSimulateSpinner.stop();
                                resolve(pollingResponse);
                            })
                            .catch(err => {
                                sendSimulateSpinner.stop();
                                reject(err);
                            });
                        }
                    } else if (data.statusCode === 400) {
                        sendSimulateSpinner.stop();
                        isPostRetryFinished = true;
                        reject(new ErrorResponse(`${callSimulationResponse.body.message}`, callSimulationResponse.body.message));
                    }
                });
            }
            if (operation.attempts() === CONSTANTS.CONFIGURATION.RETRY.SEND_SIMULATE.MAX_RETRY) {
                sendSimulateSpinner.stop();
                isPostRetryFinished = true;
                reject(new ErrorResponse(`${callSimulationResponse.body.message}`, callSimulationResponse.body.message));
            }
        });
    });
}

function pollingSimulationResult(simulationId, skillId, profile, debug) {
    return new Promise((resolve, reject) => {
        // max retry 20 times in 10 mins
        // which is a good estimate for skill status polling
        const operation = retry.operation({
            retries: CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.MAX_RETRY,
            minTimeout: CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.MIN_TIME_OUT,
            factor: CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.FACTOR
        });

        let isRetryFinished = false;
        let getSimulationResponse = null;
        operation.attempt(() => {
            if (operation.retry(isRetryFinished ? null : 'needRetry')) {
                apiWrapper.callGetSimulation(simulationId, skillId, profile, debug, (data) => {
                    let response = tools.convertDataToJsonObject(data.body);
                    getSimulationResponse = response;

                    // swallow SMAPI throttling exception. We don't want customer see throttling error, just keep trying polling.
                    if (data.statusCode === 429) {
                        isRetryFinished = false;
                    } else if (!response.hasOwnProperty('status')) {
                        isRetryFinished = true;
                        reject(new ErrorResponse(`Unable to get skill simulation result for simulation id ${simulationId}.`, response));
                    } else if (response.status === CONSTANTS.SKILL.SIMULATION_STATUS.IN_PROGRESS) {
                        isRetryFinished = false;
                    } else if (response.status === CONSTANTS.SKILL.SIMULATION_STATUS.SUCCESS){
                        isRetryFinished = true;
                        const result = response.result;
                        if (result && !result.hasOwnProperty('alexaExecutionInfo')) {
                            reject(new ErrorResponse(`Invalid response. Missing alexaExecutionInfo obejct in response for simulation id ${simulationId}.`, response));
                        } else {
                            resolve(response);
                        }
                    } else if (response.status === CONSTANTS.SKILL.SIMULATION_STATUS.FAILURE) {
                        isRetryFinished = true;
                        let errorMsg;
                        try {
                            errorMsg = response.result.error.message;
                        } catch (e) {
                            errorMsg = response;
                        }
                        reject(new ErrorResponse(errorMsg, response));
                    } else {
                        isRetryFinished = true;
                        reject(new ErrorResponse(`Invalid response for skill simulation for simulation id ${simulationId}.`, response));
                    }
                });
            }  

            if (operation.attempts() === CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.MAX_RETRY) {
                isRetryFinished = true;
                reject(new ErrorResponse(`Polling timeout. The simulation request is still in processing for simulation id ${simulationId}.`, getSimulationResponse));
            }
        });
    });
}

==> ask-cli-1.6.0.tgz: package/lib/dialog/simulation-sender.js <==
'use strict';

const apiWrapper = require('../api/api-wrapper.js');
const tools = require('../utils/tools');
const CONSTANTS = require('../utils/constants');
const displayUtil = require('../utils/display-utility');
const asyncWriter = require('./file-writer');
const retry = require('retry');
const jsonUtility = require('../utils/json-utility');

module.exports = {
    sendDialogSimulate: sendDialogSimulate
};

function ErrorResponse(msg, payload) {
    this.msg = msg;
    this.payload = payload;
    return this;
}

function sendDialogSimulate(text, skillId, locale, newSession, outputWriteStream, inputTextArray, stage, profile, debug) {
    return new Promise((resolve, reject) => {
        const sendSimulateSpinner = displayUtil.Spinner('');

        asyncWriter.writeUserSays(outputWriteStream, text);

        inputTextArray.push(text);

        const operation = retry.operation({
            retries: CONSTANTS.CONFIGURATION.RETRY.SEND_SIMULATE.MAX_RETRY,
            minTimeout: CONSTANTS.CONFIGURATION.RETRY.SEND_SIMULATE.MIN_TIME_OUT,
            factor: CONSTANTS.CONFIGURATION.RETRY.SEND_SIMULATE.FACTOR
        });

        sendSimulateSpinner.start();

        let isPostRetryFinished = false;
        let callSimulationResponse = null;
        operation.attempt(() => {
            if (operation.retry(isPostRetryFinished ? null : 'needRetry')) {
                apiWrapper.callSimulateSkill(CONSTANTS.COMMAND.NAME.DIALOG, null, text, skillId, locale, newSession, stage, profile, debug, data => {
                    callSimulationResponse = data;
                    // statusCode 400 is Bad Request which doesn't need to retry
                    // all statusCode >= 300 and not 400 will keep retry 
                    if (data.statusCode !== 400 && data.statusCode < 300) {
                        isPostRetryFinished = true;
                        const response = tools.convertDataToJsonObject(data.body);

                        if (response) {
                            pollingSimulationResult(response.id, skillId, stage, profile, debug)
                            .then(pollingResponse => {
                                sendSimulateSpinner.stop();
                                resolve(pollingResponse);
                            })
                            .catch(err => {
                                sendSimulateSpinner.stop();
                                reject(err);
                            });
                        }
                    } else if (data.statusCode === 400) {
                        sendSimulateSpinner.stop();
                        isPostRetryFinished = true;
                        reject(new ErrorResponse(`${callSimulationResponse.body.message}`, callSimulationResponse.body.message));
                    }
                });
            }
            if (operation.attempts() === CONSTANTS.CONFIGURATION.RETRY.SEND_SIMULATE.MAX_RETRY) {
                sendSimulateSpinner.stop();
                isPostRetryFinished = true;
                reject(new ErrorResponse(`${callSimulationResponse.body.message}`, callSimulationResponse.body.message));
            }
        });
    });
}

function pollingSimulationResult(simulationId, skillId, stage, profile, debug) {
    return new Promise((resolve, reject) => {
        // max retry 20 times in 10 mins
        // which is a good estimate for skill status polling
        const operation = retry.operation({
            retries: CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.MAX_RETRY,
            minTimeout: CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.MIN_TIME_OUT,
            factor: CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.FACTOR
        });

        let isRetryFinished = false;
        let getSimulationResponse = null;
        operation.attempt(() => {
            if (operation.retry(isRetryFinished ? null : 'needRetry')) {
                apiWrapper.callGetSimulation(CONSTANTS.COMMAND.NAME.DIALOG, simulationId, skillId, stage, profile, debug, (data) => {
                    let response = tools.convertDataToJsonObject(data.body);
                    getSimulationResponse = response;

                    // swallow SMAPI throttling exception. We don't want customer see throttling error, just keep trying polling.
                    if (data.statusCode === 429) {
                        isRetryFinished = false;
                    } else if (!response.hasOwnProperty('status')) {
                        isRetryFinished = true;
                        reject(new ErrorResponse(`Unable to get skill simulation result for simulation id ${simulationId}.`, response));
                    } else if (response.status === CONSTANTS.SKILL.SIMULATION_STATUS.IN_PROGRESS) {
                        isRetryFinished = false;
                    } else if (response.status === CONSTANTS.SKILL.SIMULATION_STATUS.SUCCESS){
                        isRetryFinished = true;
                        const result = response.result;
                        if (result && !result.hasOwnProperty('alexaExecutionInfo')) {
                            reject(new ErrorResponse(`Invalid response. Missing alexaExecutionInfo obejct in response for simulation id ${simulationId}.`, response));
                        } else {
                            resolve(response);
                        }
                    } else if (response.status === CONSTANTS.SKILL.SIMULATION_STATUS.FAILURE) {
                        isRetryFinished = true;
                        let errorMsg;
                        try {
                            errorMsg = response.result.error.message;
                        } catch (e) {
                            errorMsg = response;
                        }
                        reject(new ErrorResponse(errorMsg, response));
                    } else {
                        isRetryFinished = true;
                        reject(new ErrorResponse(`Invalid response for skill simulation for simulation id ${simulationId}.`, response));
                    }
                });
            }  

            if (operation.attempts() === CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.MAX_RETRY) {
                isRetryFinished = true;
                reject(new ErrorResponse(`Polling timeout. The simulation request is still in processing for simulation id ${simulationId}.`, getSimulationResponse));
            }
        });
    });
}
